@@ -9,7 +9,7 @@ def show(parent):
     dialog = QDialog(parent)
     abt = ankiqt.forms.about.Ui_About()
     abt.setupUi(dialog)
-    abt.label.setText("""
+    abt.label.setText(_("""
 <h1>Anki</h1>
 <img src=":/icons/anki.png">
 <p>
@@ -17,5 +17,5 @@ def show(parent):
 memory potential.<p/>It's free and licensed under the GPL.<p/>
 Version %s<br>
 <a href="http://ichi2.net/anki/">Visit website</a></span>
-""" % appVersion)
+""") % appVersion)
     dialog.exec_()
