@@ -9,6 +9,10 @@ import anki
 from anki.models import FieldModel, CardModel
 from ankiqt import ui
 
+tabs = ("General",
+        "Fields",
+        "Cards")
+
 class ModelProperties(QDialog):
 
     def __init__(self, parent, model, main=None, onFinish=None):
@@ -22,36 +26,48 @@ class ModelProperties(QDialog):
         self.onFinish = onFinish
         self.dialog = ankiqt.forms.modelproperties.Ui_ModelProperties()
         self.dialog.setupUi(self)
+        self.connect(self.dialog.buttonBox, SIGNAL("helpRequested()"),
+                     self.helpRequested)
         self.setupFields()
         self.setupCards()
         self.readData()
         self.show()
+        self.undoName = _("Model Properties")
+        self.parent.deck.setUndoStart(self.undoName)
+        self.exec_()
 
     def readData(self):
         # properties section
         self.dialog.name.setText(self.m.name)
-        self.dialog.description.setText(self.m.description)
         self.dialog.tags.setText(self.m.tags)
-        self.dialog.decorators.setText(self.m.features)
-        self.dialog.spacing.setText(str(self.m.spacing*100))
-        self.dialog.initialSpacing.setText(str(self.m.initialSpacing))
+        self.dialog.spacing.setText(str(self.m.spacing))
+        self.dialog.initialSpacing.setText(str(self.m.initialSpacing/60))
 
     # Fields
     ##########################################################################
 
     def setupFields(self):
+        self.fieldOrdinalUpdatedIds = []
         self.ignoreFieldUpdate = False
         self.currentField = None
         self.updateFields()
         self.readCurrentField()
         self.connect(self.dialog.fieldList, SIGNAL("currentRowChanged(int)"),
                      self.fieldRowChanged)
+        self.connect(self.dialog.tabWidget, SIGNAL("currentChanged(int)"),
+                     self.fieldRowChanged)
         self.connect(self.dialog.fieldAdd, SIGNAL("clicked()"),
                      self.addField)
         self.connect(self.dialog.fieldDelete, SIGNAL("clicked()"),
                      self.deleteField)
+        self.connect(self.dialog.fieldUp, SIGNAL("clicked()"),
+                     self.moveFieldUp)
+        self.connect(self.dialog.fieldDown, SIGNAL("clicked()"),
+                     self.moveFieldDown)
+        self.connect(self.dialog.fieldName, SIGNAL("lostFocus()"),
+                     self.updateFields)
 
-    def updateFields(self):
+    def updateFields(self, row = None):
         oldRow = self.dialog.fieldList.currentRow()
         if oldRow == -1:
             oldRow = 0
@@ -66,7 +82,14 @@ class ModelProperties(QDialog):
             item = QListWidgetItem(label)
             self.dialog.fieldList.addItem(item)
             n += 1
-        self.dialog.fieldList.setCurrentRow(oldRow)
+        count = self.dialog.fieldList.count()
+        if row != None:
+            self.dialog.fieldList.setCurrentRow(row)
+        else:
+            while (count > 0 and oldRow > (count - 1)):
+                    oldRow -= 1
+            self.dialog.fieldList.setCurrentRow(oldRow)
+        self.enableFieldMoveButtons()
 
     def fieldRowChanged(self):
         if self.ignoreFieldUpdate:
@@ -77,18 +100,29 @@ class ModelProperties(QDialog):
     def readCurrentField(self):
         if not len(self.m.fieldModels):
             self.dialog.fieldEditBox.hide()
+            self.dialog.fieldUp.setEnabled(False)
+            self.dialog.fieldDown.setEnabled(False)
             return
         else:
             self.dialog.fieldEditBox.show()
         self.currentField = self.m.fieldModels[self.dialog.fieldList.currentRow()]
         field = self.currentField
         self.dialog.fieldName.setText(field.name)
-        self.dialog.fieldDescription.setText(field.description)
         self.dialog.fieldUnique.setChecked(field.unique)
         self.dialog.fieldRequired.setChecked(field.required)
-        self.dialog.fieldFeatures.setText(field.features)
         self.dialog.numeric.setChecked(field.numeric)
 
+    def enableFieldMoveButtons(self):
+        row = self.dialog.fieldList.currentRow()
+        if row < 1:
+            self.dialog.fieldUp.setEnabled(False)
+        else:
+            self.dialog.fieldUp.setEnabled(True)
+        if row == -1 or row >= (self.dialog.fieldList.count() - 1):
+            self.dialog.fieldDown.setEnabled(False)
+        else:
+            self.dialog.fieldDown.setEnabled(True)
+
     def saveCurrentField(self):
         if not self.currentField:
             return
@@ -101,10 +135,6 @@ class ModelProperties(QDialog):
             self.deck.renameFieldModel(self.m, field, name)
             # the card models will have been updated
             self.readCurrentCard()
-        self.updateField(field, 'description',
-                         unicode(self.dialog.fieldDescription.toPlainText()))
-        self.updateField(field, 'features',
-                         unicode(self.dialog.fieldFeatures.text()))
         # unique, required, numeric
         self.updateField(field, 'unique',
                          self.dialog.fieldUnique.checkState() == Qt.Checked)
@@ -131,7 +161,8 @@ class ModelProperties(QDialog):
             return
         if len(self.m.fieldModels) < 2:
             ui.utils.showInfo(
-                _("Please add a new field first."))
+                _("Please add a new field first."),
+                parent=self)
             return
         field = self.m.fieldModels[row]
         count = self.deck.fieldModelUseCount(field)
@@ -148,10 +179,47 @@ class ModelProperties(QDialog):
         # need to update q/a format
         self.readCurrentCard()
 
+    def moveFieldUp(self):
+        row = self.dialog.fieldList.currentRow()
+        if row == -1:
+            return
+        if row == 0:
+            return
+        field = self.m.fieldModels[row]
+        tField = self.m.fieldModels[row - 1]
+        self.m.fieldModels.remove(field)
+        self.m.fieldModels.insert(row - 1, field)
+        if field.id not in self.fieldOrdinalUpdatedIds:
+            self.fieldOrdinalUpdatedIds.append(field.id)
+        if tField.id not in self.fieldOrdinalUpdatedIds:
+            self.fieldOrdinalUpdatedIds.append(tField.id)
+        self.ignoreFieldUpdate = True
+        self.updateFields(row - 1)
+        self.ignoreFieldUpdate = False
+
+    def moveFieldDown(self):
+        row = self.dialog.fieldList.currentRow()
+        if row == -1:
+            return
+        if row == len(self.m.fieldModels) - 1:
+            return
+        field = self.m.fieldModels[row]
+        tField = self.m.fieldModels[row + 1]
+        self.m.fieldModels.remove(field)
+        self.m.fieldModels.insert(row + 1, field)
+        if field.id not in self.fieldOrdinalUpdatedIds:
+            self.fieldOrdinalUpdatedIds.append(field.id)
+        if tField.id not in self.fieldOrdinalUpdatedIds:
+            self.fieldOrdinalUpdatedIds.append(tField.id)
+        self.ignoreFieldUpdate = True
+        self.updateFields(row + 1)
+        self.ignoreFieldUpdate = False
+
     # Cards
     ##########################################################################
 
     def setupCards(self):
+        self.cardOrdinalUpdatedIds = []
         self.ignoreCardUpdate = False
         self.currentCard = None
         self.updateCards()
@@ -164,8 +232,14 @@ class ModelProperties(QDialog):
                      self.deleteCard)
         self.connect(self.dialog.cardToggle, SIGNAL("clicked()"),
                      self.toggleCard)
+        self.connect(self.dialog.cardUp, SIGNAL("clicked()"),
+                     self.moveCardUp)
+        self.connect(self.dialog.cardDown, SIGNAL("clicked()"),
+                     self.moveCardDown)
+        self.connect(self.dialog.cardName, SIGNAL("lostFocus()"),
+                     self.updateCards)
 
-    def updateCards(self):
+    def updateCards(self, row = None):
         oldRow = self.dialog.cardList.currentRow()
         if oldRow == -1:
             oldRow = 0
@@ -185,7 +259,14 @@ class ModelProperties(QDialog):
             item = QListWidgetItem(label)
             self.dialog.cardList.addItem(item)
             n += 1
-        self.dialog.cardList.setCurrentRow(oldRow)
+        count = self.dialog.cardList.count()
+        if row != None:
+            self.dialog.cardList.setCurrentRow(row)
+        else:
+            while (count > 0 and oldRow > (count - 1)):
+                oldRow -= 1
+            self.dialog.cardList.setCurrentRow(oldRow)
+        self.enableCardMoveButtons()
 
     def cardRowChanged(self):
         if self.ignoreCardUpdate:
@@ -198,6 +279,8 @@ class ModelProperties(QDialog):
             self.dialog.cardEditBox.hide()
             self.dialog.cardToggle.setEnabled(False)
             self.dialog.cardDelete.setEnabled(False)
+            self.dialog.cardUp.setEnabled(False)
+            self.dialog.cardDown.setEnabled(False)
             return
         else:
             self.dialog.cardEditBox.show()
@@ -206,15 +289,23 @@ class ModelProperties(QDialog):
         self.currentCard = self.m.cardModels[self.dialog.cardList.currentRow()]
         card = self.currentCard
         self.dialog.cardName.setText(card.name)
-        self.dialog.cardDescription.setText(card.description)
         self.dialog.cardQuestion.setPlainText(card.qformat.replace("<br>", "\n"))
         self.dialog.cardAnswer.setPlainText(card.aformat.replace("<br>", "\n"))
-        if card.questionInAnswer:
-            self.dialog.questionInAnswer.setCheckState(Qt.Checked)
-        else:
-            self.dialog.questionInAnswer.setCheckState(Qt.Unchecked)
+        self.dialog.questionInAnswer.setChecked(card.questionInAnswer)
+        self.dialog.allowEmptyAnswer.setChecked(card.allowEmptyAnswer)
         self.updateToggleButtonText(card)
 
+    def enableCardMoveButtons(self):
+        row = self.dialog.cardList.currentRow()
+        if row < 1:
+            self.dialog.cardUp.setEnabled(False)
+        else:
+            self.dialog.cardUp.setEnabled(True)
+        if row == -1 or row >= (self.dialog.cardList.count() - 1):
+            self.dialog.cardDown.setEnabled(False)
+        else:
+            self.dialog.cardDown.setEnabled(True)
+
     def updateToggleButtonText(self, card):
         if card.active:
             self.dialog.cardToggle.setText(_("Disa&ble"))
@@ -229,19 +320,18 @@ class ModelProperties(QDialog):
         if not newname:
             newname = _("Card %d") % (self.m.cardModels.index(card) + 1)
         self.updateField(card, 'name', newname)
-        self.updateField(card, 'description', unicode(
-            self.dialog.cardDescription.toPlainText()))
-        s = unicode(self.dialog.cardQuestion.toPlainText()).strip()
+        s = unicode(self.dialog.cardQuestion.toPlainText())
         s = s.replace("\n", "<br>")
         changed = self.updateField(card, 'qformat', s)
-        s = unicode(self.dialog.cardAnswer.toPlainText()).strip()
+        s = unicode(self.dialog.cardAnswer.toPlainText())
         s = s.replace("\n", "<br>")
         changed2 = self.updateField(card, 'aformat', s)
         changed = changed or changed2
         self.updateField(card, 'questionInAnswer', self.dialog.questionInAnswer.isChecked())
+        self.updateField(card, 'allowEmptyAnswer', self.dialog.allowEmptyAnswer.isChecked())
         if changed:
             # need to generate all question/answers for this card
-            self.deck.updateCardsFromModel(self.currentCard)
+            self.deck.updateCardsFromModel(self.m)
         self.ignoreCardUpdate = True
         self.updateCards()
         self.ignoreCardUpdate = False
@@ -307,6 +397,48 @@ class ModelProperties(QDialog):
         self.m.setModified()
         self.deck.setModified()
 
+    def moveCardUp(self):
+        row = self.dialog.cardList.currentRow()
+        if row == -1:
+            return
+        if row == 0:
+            return
+        card = self.m.cardModels[row]
+        tCard = self.m.cardModels[row - 1]
+        self.m.cardModels.remove(card)
+        self.m.cardModels.insert(row - 1, card)
+        if card.id not in self.cardOrdinalUpdatedIds:
+            self.cardOrdinalUpdatedIds.append(card.id)
+        if tCard.id not in self.cardOrdinalUpdatedIds:
+            self.cardOrdinalUpdatedIds.append(tCard.id)
+        self.ignoreCardUpdate = True
+        self.updateCards(row - 1)
+        self.ignoreCardUpdate = False
+
+    def moveCardDown(self):
+        row = self.dialog.cardList.currentRow()
+        if row == -1:
+            return
+        if row == len(self.m.cardModels) - 1:
+            return
+        card = self.m.cardModels[row]
+        tCard = self.m.cardModels[row + 1]
+        self.m.cardModels.remove(card)
+        self.m.cardModels.insert(row + 1, card)
+        if card.id not in self.cardOrdinalUpdatedIds:
+            self.cardOrdinalUpdatedIds.append(card.id)
+        if tCard.id not in self.cardOrdinalUpdatedIds:
+            self.cardOrdinalUpdatedIds.append(tCard.id)
+        self.ignoreCardUpdate = True
+        self.updateCards(row + 1)
+        self.ignoreCardUpdate = False
+
+    def helpRequested(self):
+        idx = self.dialog.tabWidget.currentIndex()
+        QDesktopServices.openUrl(QUrl(ankiqt.appWiki +
+                                      "ModelProperties#" +
+                                      tabs[idx]))
+
     # Cleanup
     ##########################################################################
 
@@ -317,24 +449,33 @@ class ModelProperties(QDialog):
         if not mname:
             mname = _("Model")
         self.updateField(self.m, 'name', mname)
-        self.updateField(self.m, 'description',
-                         unicode(self.dialog.description.toPlainText()))
         self.updateField(self.m, 'tags',
                          unicode(self.dialog.tags.text()))
-        self.updateField(self.m, 'features',
-                         unicode(self.dialog.decorators.text()))
         try:
             self.updateField(self.m, 'spacing',
-                             float(self.dialog.spacing.text())/100.0)
+                             float(self.dialog.spacing.text()))
             self.updateField(self.m, 'initialSpacing',
-                             int(self.dialog.initialSpacing.text()))
+                             float(self.dialog.initialSpacing.text())*60)
         except ValueError:
             pass
         # before field, or it's overwritten
         self.saveCurrentCard()
         self.saveCurrentField()
+        # rebuild ordinals if changed
+        if len(self.fieldOrdinalUpdatedIds) > 0:
+            self.deck.rebuildFieldOrdinals(self.m.id, self.fieldOrdinalUpdatedIds)
+            self.m.setModified()
+            self.deck.setModified()
+        if len(self.cardOrdinalUpdatedIds) > 0:
+            self.deck.rebuildCardOrdinals(self.cardOrdinalUpdatedIds)
+            self.m.setModified()
+            self.deck.setModified()
+        # if changed, reset deck
         if self.origModTime != self.deck.modified:
             self.parent.reset()
         if self.onFinish:
             self.onFinish()
+        self.parent.deck.setUndoEnd(self.undoName)
+        # check again
+        self.parent.deck.haveJapanese = None
         QDialog.reject(self)
