@@ -7,7 +7,9 @@ import urllib, urllib2, os, sys, time, httplib
 import anki, anki.utils, anki.lang, anki.stats
 import ankiqt
 import simplejson
+import tempfile
 
+#baseUrl = "http://localhost:5000/update/"
 baseUrl = "http://anki.ichi2.net/update/"
 
 # when requesting latest version number, gather their version, deck size and
@@ -21,8 +23,8 @@ class LatestVersionFinder(QThread):
         self.config = main.config
         # calculate stats before we start a new thread
         if self.main.deck != None:
-            deckSize = self.main.deck.totalCardCount()
-            stats = anki.stats.globalStats(self.main.deck.s)
+            deckSize = self.main.deck.cardCount
+            stats = anki.stats.globalStats(self.main.deck)
             deckRecall = "%0.2f" % (
                 (stats.matureEase3 + stats.matureEase4) /
                 float(stats.matureEase0 +
@@ -30,8 +32,8 @@ class LatestVersionFinder(QThread):
                       stats.matureEase2 +
                       stats.matureEase3 +
                       stats.matureEase4 + 0.000001) * 100)
-            pending = "(%d, %d)" % (self.main.deck.oldCardCount(),
-                                    self.main.deck.newCardCount())
+            pending = "(%d, %d)" % (self.main.deck.seenCardCount(),
+                                    self.main.deck.newCount)
             ct = self.main.deck.created
             if ct:
                 ol = anki.lang.getLang()
@@ -44,7 +46,7 @@ class LatestVersionFinder(QThread):
             plat=sys.platform
             pver=sys.version
         else:
-            deckSize = "nodeck"
+            deckSize = "noDeck"
             deckRecall = ""
             pending = ""
             age = ""
@@ -83,8 +85,8 @@ class Updater(QThread):
 
     filename = "anki-update.exe"
     # FIXME: get when checking version number
-    chunkSize = 428027
-    percChange = 5
+    chunkSize = 131018
+    percChange = 1
 
     def __init__(self):
         QThread.__init__(self)
@@ -93,6 +95,8 @@ class Updater(QThread):
         self.emit(SIGNAL("statusChanged"), msg, timeout)
 
     def run(self):
+        dir = tempfile.mkdtemp(prefix="anki-update")
+        os.chdir(dir)
         filename = os.path.abspath(self.filename)
         try:
             f = urllib2.urlopen(baseUrl + "getQt")
@@ -105,9 +109,9 @@ class Updater(QThread):
             self.setStatus(_("Unable to open file"))
             return
         perc = 0
+        self.emit(SIGNAL("updateStarted"), perc)
         while 1:
-            self.setStatus(
-                _("Downloading anki updater - %d%% complete.") % perc)
+            self.emit(SIGNAL("updateDownloading"), perc)
             resp = f.read(self.chunkSize)
             if not resp:
                 break
@@ -116,38 +120,45 @@ class Updater(QThread):
             if perc > 99:
                 perc = 99
         newfile.close()
-        self.setStatus(_("Updating.."))
+        self.emit(SIGNAL("updateFinished"), perc)
         os.chdir(os.path.dirname(filename))
         os.system(os.path.basename(filename))
         self.setStatus(_("Update complete. Please restart Anki."))
         os.unlink(filename)
 
 def askAndUpdate(parent, version=None):
-    if not version:
-        baseStr = _("<h1>Update Anki manually</h1>"
-                    "Usually there is no need to do this. "
-                    "You will be notified automatically "
-                    "when a new version of Anki is available. ")
-    else:
-        baseStr = _("""<h1>Anki updated</h1>A new version of Anki is
-        available.<br/>""")
-    if sys.platform == "win32":
-        # automated installer available
-        ret = QMessageBox.question(
-            parent, "Anki", baseStr +
-            _("Would you like to download it now?"),
-            QMessageBox.Yes | QMessageBox.No)
-        if ret == QMessageBox.Yes:
+    version = version['latestVersion']
+    baseStr = (
+        _('''<h1>Anki updated</h1>Anki %s has been released.<br>
+The release notes are
+<a href="http://ichi2.net/anki/download/index.html#changes">here</a>.
+<br><br>''') %
+        version)
+    msg = QMessageBox(parent)
+    msg.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
+    msg.setIcon(QMessageBox.Information)
+    msg.setText(baseStr + _("Would you like to download it now?"))
+    button = QPushButton(_("Ignore this update"))
+    msg.addButton(button, QMessageBox.RejectRole)
+    ret = msg.exec_()
+    if msg.clickedButton() == button:
+        # ignore this update
+        parent.config['suppressUpdate'] = version
+    elif ret == QMessageBox.Yes:
+        if sys.platform == "win32":
             parent.autoUpdate = Updater()
             parent.connect(parent.autoUpdate,
                            SIGNAL("statusChanged"),
                            parent.setStatus)
+            parent.connect(parent.autoUpdate,
+                           SIGNAL("updateStarted"),
+                           parent.updateStarted)
+            parent.connect(parent.autoUpdate,
+                           SIGNAL("updateDownloading"),
+                           parent.updateDownloading)
+            parent.connect(parent.autoUpdate,
+                           SIGNAL("updateFinished"),
+                           parent.updateFinished)
             parent.autoUpdate.start()
-    else:
-        # manual
-        ret = QMessageBox.question(
-            parent, "Anki", baseStr +
-            "Would you like to visit the website now?",
-            QMessageBox.Yes | QMessageBox.No)
-        if ret == QMessageBox.Yes:
+        else:
             QDesktopServices.openUrl(QUrl(ankiqt.appWebsite))
