@@ -12,9 +12,9 @@ import itertools, time
 from operator import itemgetter
 from anki import DeckStorage
 from anki.cards import Card
-from anki.sync import SyncClient, SyncServer
+from anki.sync import SyncClient, SyncServer, BulkMediaSyncer
 from anki.lang import _
-from anki.utils import findTag, parseTags, stripHTML
+from anki.utils import findTag, parseTags, stripHTML, ids2str
 from anki.db import *
 
 class Exporter(object):
@@ -49,7 +49,6 @@ class Exporter(object):
                 return True
         return False
 
-# FIXME: media support
 class AnkiExporter(Exporter):
 
     key = _("Anki decks (*.anki)")
@@ -63,21 +62,24 @@ class AnkiExporter(Exporter):
         self.newDeck = DeckStorage.Deck(path)
         client = SyncClient(self.deck)
         server = SyncServer(self.newDeck)
+        client.setServer(server)
         client.localTime = self.deck.modified
         client.remoteTime = 0
         self.deck.s.flush()
         # set up a custom change list and sync
         lsum = self.localSummary()
         rsum = server.summary(0)
-        payload = client.genPayload(lsum, rsum)
+        payload = client.genPayload((lsum, rsum))
         res = server.applyPayload(payload)
         client.applyPayloadReply(res)
         if not self.includeSchedulingInfo:
             self.newDeck.s.statement("""
+delete from reviewHistory""")
+            self.newDeck.s.statement("""
 update cards set
 interval = 0,
 lastInterval = 0,
-due = 0,
+due = created,
 lastDue = 0,
 factor = 2.5,
 firstAnswered = 0,
@@ -97,31 +99,42 @@ matureEase3 = 0,
 matureEase4 = 0,
 yesCount = 0,
 noCount = 0,
+spaceUntil = 0,
+isDue = 1,
+type = 2,
+combinedDue = created,
 modified = :now
 """, now=time.time())
-            # update spacing
             self.newDeck.s.statement("""
-update facts set
-lastCardId = null,
-spaceUntil = 0,
-modified = :now""", now=time.time())
+delete from stats""")
+        # media
+        if client.mediaSyncPending:
+            bulkClient = BulkMediaSyncer(client.deck)
+            bulkServer = BulkMediaSyncer(server.deck)
+            bulkClient.server = bulkServer
+            bulkClient.sync()
         # need to save manually
+        self.newDeck.rebuildCounts()
+        self.exportedCards = self.newDeck.cardCount
         self.newDeck.s.commit()
         self.newDeck.close()
 
     def localSummary(self):
         cardIds = self.cardIds()
+        cStrIds = ids2str(cardIds)
         cards = self.deck.s.all("""
 select id, modified from cards
-where id in (%s)""" % ",".join([str(c) for c in cardIds]))
+where id in %s""" % cStrIds)
         facts = self.deck.s.all("""
 select facts.id, facts.modified from cards, facts where
 facts.id = cards.factId and
-cards.id in (%s)""" % ",".join([str(c) for c in cardIds]))
+cards.id in %s""" % cStrIds)
         models = self.deck.s.all("""
 select models.id, models.modified from models, facts where
 facts.modelId = models.id and
-facts.id in (%s)""" % ",".join([str(f[0]) for f in facts]))
+facts.id in %s""" % ids2str([f[0] for f in facts]))
+        media = self.deck.s.all("""
+select id, created from media""")
         return {
             # cards
             "cards": cards,
@@ -132,6 +145,9 @@ facts.id in (%s)""" % ",".join([str(f[0]) for f in facts]))
             # models
             "models": models,
             "delmodels": [],
+            # media
+            "media": media,
+            "delmedia": [],
             }
 
 class TextCardExporter(Exporter):
@@ -144,15 +160,15 @@ class TextCardExporter(Exporter):
         self.includeTags = False
 
     def doExport(self, file):
-        strids = ",".join([str(id) for id in self.cardIds()])
+        strids = ids2str(self.cardIds())
         cards = self.deck.s.all("""
 select cards.question, cards.answer, cards.id from cards
-where cards.id in (%s)""" % strids)
+where cards.id in %s""" % strids)
         if self.includeTags:
             self.cardTags = dict(self.deck.s.all("""
 select cards.id, cards.tags || "," || facts.tags from cards, facts
 where cards.factId = facts.id
-and cards.id in (%s)""" % strids))
+and cards.id in %s""" % strids))
         out = u"\n".join(["%s\t%s%s" % (self.escapeText(c[0]),
                                         self.escapeText(c[1]),
                                         self.tags(c[2]))
@@ -183,13 +199,13 @@ where
 factId in
 (select distinct facts.id from facts, cards
 where facts.id = cards.factId
-and cards.id in (%s))
-order by factId, ordinal""" % ",".join([str(s) for s in cardIds]))
+and cards.id in %s)
+order by factId, ordinal""" % ids2str(cardIds))
         txt = ""
         if self.includeTags:
             self.factTags = dict(self.deck.s.all(
-                "select id, tags from facts where id in (%s)" %
-                ",".join([str(fact[0]) for fact in facts])))
+                "select id, tags from facts where id in %s" %
+                ids2str([fact[0] for fact in facts])))
         out = ["\t".join([self.escapeText(x[1]) for x in ret[1]]) +
                self.tags(ret[0])
                for ret in (itertools.groupby(facts, itemgetter(0)))]
@@ -205,8 +221,8 @@ order by factId, ordinal""" % ",".join([str(s) for s in cardIds]))
 # Export modules
 ##########################################################################
 
-Exporters = (
-    (_("Anki deck (*.anki)"), AnkiExporter),
-    (_("Cards in tab-separated text file (*.txt)"), TextCardExporter),
-    (_("Facts in tab-separated text file (*.txt)"), TextFactExporter),
-    )
+def exporters():
+    return (
+        (_("Anki deck (*.anki)"), AnkiExporter),
+        (_("Cards in tab-separated text file (*.txt)"), TextCardExporter),
+        (_("Facts in tab-separated text file (*.txt)"), TextFactExporter))
