@@ -5,7 +5,12 @@
 """\
 History - keeping a record of all reviews
 ==========================================
+
+If users run 'check db', duplicate records will be inserted into the DB - I
+really should have used the time stamp as the key. You can remove them by
+keeping the lowest id for any given timestamp.
 """
+
 __docformat__ = 'restructuredtext'
 
 import time
@@ -45,4 +50,26 @@ class CardHistoryEntry(object):
         self.delay = delay
         self.thinkingTime = card.thinkingTime()
 
+    def writeSQL(self, s):
+        s.statement("""
+insert into reviewHistory
+(cardId, lastInterval, nextInterval, ease, delay, lastFactor,
+nextFactor, reps, thinkingTime, yesCount, noCount, time)
+values (
+:cardId, :lastInterval, :nextInterval, :ease, :delay,
+:lastFactor, :nextFactor, :reps, :thinkingTime, :yesCount, :noCount,
+:time)""",
+                    cardId=self.cardId,
+                    lastInterval=self.lastInterval,
+                    nextInterval=self.nextInterval,
+                    ease=self.ease,
+                    delay=self.delay,
+                    lastFactor=self.lastFactor,
+                    nextFactor=self.nextFactor,
+                    reps=self.reps,
+                    thinkingTime=self.thinkingTime,
+                    yesCount=self.yesCount,
+                    noCount=self.noCount,
+                    time=time.time())
+
 mapper(CardHistoryEntry, reviewHistoryTable)
