@@ -2,8 +2,9 @@
 
 cd /Volumes/Two/anki
 echo "syncing updates..."
-rsync -av reflex:Lib/code/libanki --exclude .hg --exclude build --exclude dist --delete .
-rsync -av reflex:Lib/code/ankiqt --exclude .hg --exclude build --exclude dist  --delete .
+rsync -av reflex:Lib/code/libanki --exclude .git --exclude build --exclude dist --delete .
+rsync -av reflex:Lib/code/ankiqt --exclude .git --exclude build --exclude dist  --delete .
+find . -name '*.pyc' -exec rm {} \;
 echo "updating kakasi..."
 mkdir -p kakasi
 cp -Rvf /usr/local/bin/kakasi kakasi
